<?php
/*
 * the home page
 */
require_once("include.php");
require_once("postgresql.php");
require_once("authorized_hosts.php");
require_once("app-classes/innovaphone.pbx.class.php");

$version = file_get_contents("version");

InputPage::$method = "post";

class ButtonDownloadCover extends ButtonHidden {

    public function ButtonDownloadCover($name, $label) {
        parent::__construct($name, $label);
    }

    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession*/ global $faxSession;
        $file = $faxSession->faxServer->coverPath("", $page->coverName);
        $size = filesize($file);
        $fileName = $page->coverName;
        if($file == FaxConfig::coverDefault) {
            $fileName = str_replace("/var/www/innovaphone/apps/innovaphone-faxserver/", "", FaxConfig::coverDefault);
        }
        header('Pragma: private');
        header('Cache-control: private, must-revalidate');
        header("Content-Disposition: attachment; filename=\"$fileName\"");
        header('Content-type: text');
        header('Content-len: ' . $size);
        readfile($file);
    }
}

class ButtonAskDeleteServer extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteServer($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=".FaxConfigConsts::modeDeleteServer);
    }
}

class ButtonDeleteServer extends MainInputPageOkCancelButton {
    public function ButtonDeleteServer($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession*/global $faxSession;
        $faxDb = get_fax_db();
        if($faxDb->deleteFaxServer($page->serverId) === false) {
            return $this->reRenderPage($page, $faxDb->error);
        }
        $covers = $faxSession->faxServer->getCovers();
        foreach($covers as $cover) {
            if($cover != FaxConfig::coverDefault) {
                unlink($faxSession->faxServer->coverPath("", $cover));
            }
        }
        $faxSession->faxServer->stop();
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&tab=".FaxConfigConsts::tabServer."&server_id=0&serverdeleted=".urlencode($page->serverName));
    }
    
    function reRenderPage(&$page, $error) {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonDeleteCover extends MainInputPageOkCancelButton {
    public function ButtonDeleteCover($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession*/ global $faxSession;
        /* @var $faxServer FaxServer*/ $faxServer = $faxSession->faxServer;
        if($page->coverName == FaxConfig::coverDefault) {
            return $this->reRenderPage($page, tl::tl("You can't delete the default cover!"));
        }
        if($faxServer->cover == $page->coverName) {
            return $this->reRenderPage($page, tl::tl("You can't delete this cover, as it is used by this server!"));
        }
        foreach($faxServer->locations as $loc) {
            if($loc->cover == $page->coverName) {
                return $this->reRenderPage($page, tl::tlx('You can\'t delete this cover, as it is used by the location {$arg[1]}!', $loc->name));
            }
        }
        unlink($faxServer->coverPath("", $page->coverName));
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=".$this->prevMode."&coverdeleted=".urlencode($page->coverName));
    }
    
    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonAddCover extends MainInputPageOkCancelButton {
    public function ButtonAddCover($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession*/ global $faxSession;
        $file = $page->getField("cover");
        $tmpPath = $faxSession->faxServer->coverPath("", $file->status->name);
        if($tmpPath == FaxConfig::coverDefault) {
            return $this->reRenderPage($page, tl::tlx('File {$arg[1]} can\'t be overwritten!', $file->status->name));
        }
        $zip = new ZipArchive();
        if($zip->open($file->status->path) !== true) {
            return $this->reRenderPage($page, tl::tlx('File {$arg[1]} couldn\'t be validated!', $file->status->name));
        }
        $tmpDir = tempdir(FaxConfig::tmpPath, "content");
        $ret = $zip->extractTo($tmpDir, "content.xml");
        $content = "";
        if($ret !== false) {
            $content = file_get_contents($tmpDir."/content.xml");
        }
        $zip->close();
        system("rm -R \"$tmpDir\"");
        if($ret === false) {
           return $this->reRenderPage($page, tl::tl("Couldn't extract content.xml from file. No Open Document Format!"));
        }
        $doc = new DOMDocument();
        if($doc->loadXML($content) === false) {
            return $this->reRenderPage($page, tl::tl("Couldn't validate content.xml from file."));
        }
        move_uploaded_file($file->status->path, $faxSession->faxServer->coverPath("", $file->status->name));
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=".$this->prevMode."&coveradded=".urlencode($page->coverName));
    }
    
    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonServerStrings1 extends MainInputPageOkCancelButton {
    public function ButtonServerStrings1($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
        $lang = $page->getField("stringlang");
        $faxSession->faxServer->stringLanguage = array_search("true", $lang->value);
        $faxSession->updateServer($faxSession->faxServer);
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=".$this->nextMode);
    }
}

class ButtonServerStrings2 extends MainInputPageOkCancelButton {
    public function ButtonServerStrings2($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
        /* @var $faxServer FaxServer */ $faxServer = $faxSession->faxServer;
        $list = $page->getField("liststrings");
        $faxServer->strings[$faxServer->stringLanguage] = array();
        foreach($list->value as $value) {
            $val = str_replace("\r\n", "\n", $value[$list->name."value"]);
            $id = intval($value[$list->name."id"]);
            if($val != "" && $val != $faxServer->getString($id, $faxServer->stringLanguage, false)) {
                $faxServer->strings[$faxServer->stringLanguage][$id] = $val;
            }
        }
        if($faxServer->writeStrings($faxServer->stringLanguage, get_fax_db()) === false) {
            return $this->reRenderPage($page, $faxServer->error);
        }
        $faxServer->restart();
        $faxSession->updateServer($faxSession->faxServer);
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&stringsupdated=1");
        exit;
    }
    
    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonServerConfig1 extends MainInputPageOkCancelButton {
    public function ButtonServerConfig1($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
        $name = $page->getField("name");
        $desc = $page->getField("desc");
        $lang = $page->getField("faxlang");
        $list = $page->getField("listlocations");
        $trace = $page->getField("trace");
        $active = $page->getField("active");
        $cover = $page->getField("cover");
        
        $server = $faxSession->faxServer;
        $server->name = $name->value;
        $server->desc = $desc->value;
        $server->lang = array_search("true", $lang->value);
        $server->trace = ($trace->value["on"] ? true : false);
        $server->active = ($active->value["on"] ? true : false);
        $server->cover = array_search("true", $cover->value);
        
        $faxDb = get_fax_db();
        if($faxDb->checkFaxServerName($server) === false) {
            return $this->reRenderPage($page, $faxDb->error);
        }
        
        $server->locations = array();
        
        foreach($list->value as $value) {
            if($value[$list->name."name"] != "") {
                $server->locations[] = new FaxLocation($value[$list->name."name"], 
                                                        array_search("true", $value[$list->name."lang"]),
                                                        array_search("true", $value[$list->name."cover"]));
            }
        }
        
        $faxSession->updateServer($server);
        
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=".$this->nextMode);
    }

    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonServerConfig2 extends MainInputPageOkCancelButton {
    public function ButtonServerConfig2($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
        $host = $page->getField("mailhost");
        $port = $page->getField("mailport");
        $user = $page->getField("mailuser");
        $pw = $page->getField("mailpw");
        $folder = $page->getField("mailfolder");
        $faxDomain = $page->getField("faxdomain");
        $admin = $page->getField("mailadmin");
        $mailSAPCode = $page->getField("mailsapcode");

        if(($port->value != "" && !$port->valid()) ||
           !$user->valid())
        {
            $page->render();
            exit;
        }

        $server = $faxSession->faxServer;
        $server->mailHost = $host->value;
        $server->mailPort = $port->value;
        $server->mailUser = $user->value;
        $server->mailPassword = $pw->value;
        $server->mailFolder = $folder->value;
        $server->faxServerDomain = strtolower($faxDomain->value);
        $server->mailAdmin = strtolower($admin->value);
        $server->mailSAPCode = ($mailSAPCode->value["on"] ? true : false);
     
        $faxDb = get_fax_db();
        if($faxDb->checkFaxServerMailUser($server) === false) {
            return $this->reRenderPage($page, $faxDb->error);
        }
        
        $faxSession->updateServer($server);
        $mode = "";
        if($this->prevButton != null) {
            $mode = $this->nextMode;
        }
        else {
            $mode = $this->prevMode;
        }
        $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=$mode");
    }

    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonServerConfig3 extends MainInputPageOkCancelButton {
    public function ButtonServerConfig3($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
            
        if(PageConfig::saveValuesPage3($page)) {
            $mode = "";
            if($this->prevButton != null) {
                $mode = $this->nextMode;
            }
            else {
                $mode = $this->prevMode;
            }
            $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode=$mode");
        }
        else {
            $page->render();
        }
    }

    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonServerConfig4 extends MainInputPageOkCancelButton {
    public function ButtonServerConfig4($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
        $server = $faxSession->faxServer;
        if($server->name == null || $server->name == "") {
            home_die(tl::tl("Session exceeded!"), "config_faxserver.php");
        }
        $subscriberId = $page->getField("subscriberid");
        $subscriberIdAlt = $page->getField("subscriberidalt");
        $addUserNumber = $page->getField("addusernumber");
        $userNumberPrefix = $page->getField("usernumberprefix");
        $subscriberName = $page->getField("subscribername");
        $maxCallTries = $page->getField("maxcalltries");
        $resendDelay = $page->getField("resenddelay");
        $responseTimeout = $page->getField("responsetimeout");
        $retryTimeout = $page->getField("retrytimeout");
        $forwardedFileFormat = $page->getField("forwardedfileformat");
        $coverIncludesUserText = $page->getField("coverincludesusertext");
        $confirmationWithDoc = $page->getField("confirmationwithdoc");
        $modemSpeed = $page->getField("modemspeed");
        
        if($subscriberId->value != "" && !$subscriberId->valid()) {
            $page->render();
            exit;
        }
        if($subscriberIdAlt->value != "" && !$subscriberIdAlt->valid()) {
            $page->render();
            exit;
        }
        if($server->name == "") {   // might happen on a second Button-Ok-Click
            $page->render();
            exit;
        }
        $server->faxSubscriberId = $subscriberId->value;
        $server->faxSubscriberIdAlt = $subscriberIdAlt->value;
        $server->faxAddUserNumber = ($addUserNumber->value["on"] ? true : false);
        $server->faxUserNumberPrefix = $userNumberPrefix->value;
        $server->faxSubscriberName = $subscriberName->value;
        $server->faxMaxCallTries = intval($maxCallTries->value);
        $server->faxResendDelay = intval($resendDelay->value);
        $server->faxResponseTimeout = intval($responseTimeout->value);
        $server->faxRetryTimeout = intval($retryTimeout->value);
        $server->setForwardedFileFormat(array_search(true, $forwardedFileFormat->value));
        $server->faxCoverIncludesUserText = ($coverIncludesUserText->value["on"] ? true : false);
        $server->faxConfirmationWithDoc = ($confirmationWithDoc->value["on"] ? true : false);
        $server->setModemSpeed(array_search(true, $modemSpeed->value));
        
        $faxSession->updateServer($server);

        if($this->prevButton != null) {
            if($server->write(get_fax_db()) === false) {
                return $this->reRenderPage($page, $server->error);
            }
            $server->restart();
            $arg = "";
            if($page->serverId === 0) {
                $arg = "&createdserver=".urlencode($server->name);
            }
            else {
                $arg = "&editedserver=".urlencode($server->name);
            }
            $serverId = $page->getField("server_id");
            $serverId->value = $page->serverId = 0;
            $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs().$arg);
        }
        else {
            $page->warp(InputPage::relativeScriptName()."?_page=".FaxConfigConsts::pageConfig."&".$page->getArgs()."&mode={$this->prevMode}");
        }
    }

    function reRenderPage(InputPage &$page, $error = "") {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonTestPbx extends InputPageButtonOnlyAction {
    public function ButtonTestPbx($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $faxSession FaxSession */ global $faxSession;
        /* @var $version Version */ global $version;
        $server = $faxSession->faxServer;
        
        if(PageConfig::saveValuesPage3($page)) {
            $pbx = new InnovaphonePBX(
                $server->soapHost,
                $server->soapPort,
                $server->https,
                $server->soapUser,
                $server->soapPassword,
                $server->soapFaxUser,
                "innovaphone PHP connection test [" . $version . "]",
                array("trace" => 1),
                dirname(__FILE__) . '/wsdl/pbx10_00.wsdl',
                $server->soapDynPbx);
                
            if($pbx->getKey() == 0) {
                $this->msg = tl::tl("Error:") . " " . $pbx->getFaultString();
            }
            else {
                $userHandle = 0;
                try {
                    $userHandle = $pbx->UserInitialize($pbx->getSession(), $server->soapFaxUser, true, false, "");
                }
                catch(Exception $e) {
                    $this->msg = tl::tl("Error:") . " " . $pbx->getFaultString();
                }
                if(!$userHandle) {
                    $this->msg = tl::tl("Error:") . " " . tl::tlx("Could not subscribe to user {$arg[1]}.", $userName);
                }
                else {
                    $this->msg = tl::tl("Ok") . "!";    
                }
                try {
                    $p = $pbx->End($pbx->getSession());
                }
                catch(Exception $e) {
                }
            }
        }
        $page->render();
        $this->msg = "";
    }
}

class PageConfig extends PostgresPage {
    const btnDownloadName = "btndownload";
    
    public function getHelpAnchor() {
        switch($this->parent->currentTab) {
            case FaxConfigConsts::tabPgsql: 
                switch($this->parent->currentMode) {
                    case PostgresPage::modeInRePgPw:
                        return "#Password";
                    default: return "#Database";
                }
            case FaxConfigConsts::tabServer:
                switch($this->parent->currentMode) {
                    case FaxConfigConsts::modeDeleteServer:
                        return "#Delete_Server";
                    case FaxConfigConsts::modeServer1:
                        return "#Server_Configuration";
                    case FaxConfigConsts::modeServer2:
                        return "#Mail_Configuration";
                    case FaxConfigConsts::modeServer3:
                        return "#PBX_Configuration";
                    case FaxConfigConsts::modeServer4:
                        return "#Fax_Configuration";
                    case FaxConfigConsts::modeServerCovers:
                        return "#Covers";
                    case FaxConfigConsts::modeServerCoversAdd:
                        return "#Add_Cover";
                    case FaxConfigConsts::modeServerCoversDelete:
                        return "#Delete_Cover";
                    case FaxConfigConsts::modeServerStrings1:
                        return "#String_Language";
                    case FaxConfigConsts::modeServerStrings2:
                        return "#Configure_Strings";
                }
                break;
            case FaxConfigConsts::tabAuthorizedHosts:
                switch($this->parent->currentMode) {
                    case FaxConfigConsts::modeAuthorizedHostsDefault:
                    default:
                        return "#Authorized_Hosts";
                }
                break;
            default: return "";
        }
    }
    
    public function getDefaultTab() {
        return FaxConfigConsts::tabServer;
    }
    
    public function getHead() {
        return "";
    }
    
    public function createFields() {
        $this->addSection(FaxConfigConsts::tabServer);
        $this->addSection(FaxConfigConsts::tabAuthorizedHosts);
        $this->addSection(FaxConfigConsts::tabPgsql);
        $genericHint = "";
        
        $this->parent->addTabMenuItem(tl::tl("Server"), "javascript:submitTab('".FaxConfigConsts::tabServer."')", $this->parent->currentTab == FaxConfigConsts::tabServer);
        $this->parent->addTabMenuItem(tl::tl("Authorized Hosts"), "javascript:submitTab('".FaxConfigConsts::tabAuthorizedHosts."')", $this->parent->currentTab == FaxConfigConsts::tabAuthorizedHosts);
        $this->parent->addTabMenuItem(tl::tl("Database"), "javascript:submitTab('".FaxConfigConsts::tabPgsql."')", $this->parent->currentTab == FaxConfigConsts::tabPgsql);
        
        switch($this->parent->currentTab) {
            case FaxConfigConsts::tabServer:
                $this->tabServer();
                break;
            case FaxConfigConsts::tabAuthorizedHosts:
                AuthorizedHostsTab::createContent($this, $this->parent->currentMode);
                break;
            case FaxConfigConsts::tabPgsql:
                $this->tabPgSql("innovaphone-faxserver", FaxConfigConsts::pageConfig);
                break;
        }
    }
    
    function addActionText() {
        $text = $this->addField(new InputPageText("actiontext_{$this->currentSection}" , ""));
        $text->autorender = false;
        $text->fullline = true;
        $createdServer = InputPage::formInput("createdserver");
        $editedServer = InputPage::formInput("editedserver");
        $deletedServer = InputPage::formInput("serverdeleted");
        $stringsUpdated = InputPage::formInput("stringsupdated");
        $coverDeleted = InputPage::formInput("coverdeleted");
        $coverAdded = InputPage::formInput("coveradded");
        if(isset($createdServer )) {
            $text->value = tl::tlx('Server {$arg[1]} successfully created.', "<b>".htmlentities_utf8($createdServer)."</b>");
        }
        elseif(isset($editedServer )) {
            $text->value = tl::tlx('Server {$arg[1]} successfully edited.', "<b>".htmlentities_utf8($editedServer)."</b>");
        }
        elseif(isset($deletedServer)) {
            $text->value = tl::tlx('Server {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($deletedServer)."</b>");
        }
        elseif(isset($stringsUpdated)) {
            $text->value = tl::tl('Texts successfully edited.');
        }
        elseif(isset($coverDeleted)) {
            $text->value = tl::tlx('Cover {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($coverDeleted)."</b>");
        }
        elseif(isset($coverAdded)) {
            $text->value = tl::tlx('Cover {$arg[1]} successfully added.', "<b>".htmlentities_utf8($coverAdded)."</b>");
        }
        if($text->value != "") {
            $text->autorender = true;
            $text->value = "<i>{$text->value}</i><p>&nbsp;</p>";
        }
    }
    
    public function tabServer() {
        global $faxSession;
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        
        $error = InputPage::formInput("error");
        if (isset($error)) {
            $errorField = $this->addField(new InputPageText("error", ""));
            $errorField->value = "<b>" . tl::tl("An unexpected error occurred:") . "</b>";
            $errorField->value .= "<br><div style=\"color:red\">$error</div><br><br>";
            $errorField->fullline = true;
        }
        $error = MainMethods::checkRunningScript(Config::configErrorPostgres);
        if($error != "") {
            $running = $this->addField(new InputPageText("procrunning", $error));
            $running->fullline = true;
            return;
        }
        $createName = tl::tl("Add");
        $create = $this->addField(new InputPageText("createserverlink", $this->getModeLink(FaxConfigConsts::modeServer1, tl::tl("Add"), "", "javascript:submitServer('0', '".FaxConfigConsts::modeServer1."')")));
        $create->fullline = true;
        switch($this->parent->currentMode) {
            case FaxConfigConsts::modeServer1:
                $this->modeServer1();
                break;
            case FaxConfigConsts::modeServer2:
                $this->modeServer2();
                break;
            case FaxConfigConsts::modeServer3:
                $this->modeServer3();
                break;
            case FaxConfigConsts::modeServer4:
                $this->modeServer4();
                break;
            case FaxConfigConsts::modeServerStrings1:
                $this->modeServerStrings1();
                break;
            case FaxConfigConsts::modeServerStrings2:
                $this->modeServerStrings2();
                break;
            case FaxConfigConsts::modeDeleteServer:
                $this->modeDeleteServer();
                break;
            case FaxConfigConsts::modeServerCovers:
                $this->modeServerCovers();
                break;
            case FaxConfigConsts::modeServerCoversAdd:
                $this->modeServerCoversAdd();
                break;
            case FaxConfigConsts::modeServerCoversDelete:
                $this->modeServerCoversDelete();
                break;
        }
        if($this->parent->currentMode == "") {
            $this->addServerList();
        }
    }
    
    function addServerList() {
        // fields below the green field, here the filter list
        $this->addSection(FaxConfigConsts::tabServer."_bottom", 1, false);
        $this->currentSection = FaxConfigConsts::tabServer."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listserver"));
        $emptyText = tl::tl("No fax servers yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $list->addColumn($name = new InputPageText($list->name."name", null), tl::tl("Server"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($desc = new InputPageText($list->name."desc", null), tl::tl("Description"));
        $desc->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($strings = new InputPageText($list->name."strings", null), "");
        $strings->sortType = genericInputPageItem::sortIgnore;
        $list->addColumn($covers = new InputPageText($list->name."covers", null), "");
        $covers->sortType = genericInputPageItem::sortIgnore;
        $list->addColumn($id = new InputPageHiddenIdField($list->name."id"));
    }
    
    function modeServerCovers() {
        $this->currentSection = FaxConfigConsts::modeServerCovers;
        $this->addSection($this->currentSection, 1, true, tl::tl("Configure Server Covers"));
        /* @var $faxSession FaxSession*/global $faxSession;
        $addName = tl::tl("Add");
        $add = $this->addField(new InputPageText("addcoverlink", $this->getModeLink(FaxConfigConsts::modeServerCoversAdd, tl::tl("Add"), "", "javascript:submitServer({$faxSession->faxServer->id}, '".FaxConfigConsts::modeServerCoversAdd."')")));
        $add->fullline = true;
        $add->attributes["style"] = "margin-bottom:20px";
        
        $list = $this->addField(new MainInputPageListField("listcovers"));
        $emptyText = tl::tl("No covers yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $list->addColumn($name = new InputPageText($list->name."name", null), tl::tl("Cover"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($name = new InputPageText($list->name."delete", null), null);
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($name = new InputPageText($list->name."download", null), null);
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        
        $this->addField($btn = new ButtonCancel("btncancel", tl::tl("Close"), FaxConfigConsts::pageConfig));
        $btn->autorender = true;
        $btnDownload = $this->addField(new ButtonDownloadCover(self::btnDownloadName, ""));
    }
    
    function modeServerCoversAdd() {
        $this->currentSection = FaxConfigConsts::modeServerCoversAdd;
        $this->addSection($this->currentSection, 1, true, tl::tl("Upload a new cover"));
        $this->addField($text = new InputPageText("covertext", tl::tl("Only documents in the Open Document Format (odt) are supported!")));
        $this->addField($file = new InputPageFileUploadField("cover", tl::tl("Cover")));
        $this->addField($add = new ButtonAddCover("buttonaddcover", FaxConfigConsts::pageConfig, $this->parent, "", FaxConfigConsts::modeServerCovers));
    }
    
    function modeServerCoversDelete() {
        $this->currentSection = FaxConfigConsts::modeServerCoversDelete;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected cover \'{$arg[1]}\'?', htmlentities_utf8($this->parent->coverName)));
        $this->addField($del = new ButtonDeleteCover("buttondeletecover", FaxConfigConsts::pageConfig, $this->parent, "", FaxConfigConsts::modeServerCovers));
    }
    
    function modeDeleteServer() {
        $this->currentSection = FaxConfigConsts::modeDeleteServer;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected Fax server \'{$arg[1]}\'?', htmlentities_utf8($this->parent->serverName)));
        $this->addField($del = new ButtonDeleteServer("buttondeleteserver", FaxConfigConsts::pageConfig, $this->parent, "", FaxConfigConsts::modeServer1));
    }
    
    function modeServerStrings1() {
        /* @var $faxSession FaxSession */global $faxSession;
        $this->currentSection = FaxConfigConsts::modeServerStrings1;
        $this->addSection($this->currentSection, 1, true, tl::tl('Select the language you want to configure'));
        $this->addField($lang = new InputPageDropdownField("stringlang", tl::tl("Language"), tl::$iso639codes));
        $lang->default[$faxSession->faxServer->stringLanguage] = true;
        $this->addSubSection(FaxConfigConsts::modeServerStrings1, "_btn");
        $this->addField($btn = new ButtonServerStrings1("buttonserverstrings1", FaxConfigConsts::pageConfig, $this->parent, FaxConfigConsts::modeServerStrings2));
    }
    
    function modeServerStrings2() {
        $this->currentSection = FaxConfigConsts::modeServerStrings2;
        $this->addSection($this->currentSection, 1, true, tl::tl('Configure texts'));
        
        $list = $this->addField(new MainInputPageListField("liststrings"));
        $list->addColumn($name = new InputPageText($list->name."name", null), tl::tl("Text type"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($value = new InputPageTextareaField($list->name."value", null), tl::tl("Text"));
        $value->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $value->attributes["cols"] = 70;
        $value->attributes["rows"] = 6;
        $list->addColumn($id = new InputPageHiddenIdField($list->name."id", null), null);
        
        $this->addField($btn = new ButtonServerStrings2("buttonserverstrings2", FaxConfigConsts::pageConfig, $this->parent, "", FaxConfigConsts::modeServerStrings1));
    }
    
    function modeServer1() {
        /* @var $faxSession FaxSession */global $faxSession;
        $this->currentSection = FaxConfigConsts::modeServer1;
        $this->addSection($this->currentSection, 1, true, tl::tl("Server Configuration"));
        
        /* @var $server FaxServer */ $server = $faxSession->faxServer;
        $this->addField($active = new InputPageCheckboxField("active", tl::tl("Active")));
        $active->hint = tl::tl("If ticked, this fax server is active. Otherwise it is not started.");
        $this->addField($name = new InputPageStringField("name", tl::tl("Server Name"), $server->name));
        $name->setJavaScriptInit("function JavaScriptInit_{$name->name}() { setFocus('{$name->name}'); }");
        $this->addField($desc = new InputPageStringField("desc", tl::tl("Description"), $server->desc));
        $desc->emptyOk = true;
        $faxDb = get_fax_db();
        $this->addField($lang = new InputPageDropdownField("faxlang", tl::tl("Language"), $faxDb->getLanguages($server)));
        $lang->hint = tl::tl("The language of user and administrator response emails.");
        $lang->default[$server->lang] = true;
        $this->addField($cover = new InputPageDropdownField("cover", tl::tl("Cover"), $server->getCovers(true)));
        $cover->hint = tl::tl("A cover, which will be used, if plain text in the mail body is sent.");
        $cover->default[$server->cover] = true;
        $this->addField($trace = new InputPageCheckboxField("trace", tl::tl("Trace")));
        $trace->hint = tl::tl("If ticked, the FAX interface trace on the innovaphone device is enabled.");
        $descValue = InputPage::formInput($desc->name);
        if(!isset($descValue)) {
            $trace->default["on"] = $server->trace;
            $active->default["on"] = $server->active;
        }
        
        $this->addField(new InputPageHorizontalRule());
        
        $this->addField($list = new MainInputPageListField("listlocations"));
        $list->sortDirection = InputPageListField::sortNone;
        $list->addColumn($listName = new InputPageListStringField($list->name."name", null), tl::tl("PBX Name"));
        $listName->hint = tl::tl("The PBX name of a slave PBX, which shall have a different language/cover.");
        $listName->sortType = genericInputPageItem::sortIgnore;
        $listName->emptyOk = true;
        $list->addColumn($listLang = new InputPageDropdownField($list->name."lang", null, array()), tl::tl("Language"));
        $listLang->sortType = genericInputPageItem::sortIgnore;
        $list->addColumn($listCover = new InputPageDropdownField($list->name."cover", null, array()), tl::tl("Cover"));
        $listLang->sortType = genericInputPageItem::sortIgnore;
        $list->removableRows = true;
        
        $this->addField($btn = new ButtonServerConfig1("saveserverconfig", FaxConfigConsts::pageConfig, $this->parent, FaxConfigConsts::modeServer2));
        if($this->parent->serverId > 0) {
            $del = $this->addField(new ButtonAskDeleteServer("buttondeleteserver", tl::tl("Delete")));
        }
        $name->setButtons($btn->name, $btn->name."_cancel");
        $desc->setButtons($btn->name, $btn->name."_cancel");
        $lang->setButtons($btn->name, $btn->name."_cancel");
        $trace->setButtons($btn->name, $btn->name."_cancel");
        $active->setButtons($btn->name, $btn->name."_cancel");
        $cover->setButtons($btn->name, $btn->name."_cancel");
        $listName->setButtons($btn->name, $btn->name."_cancel");
        $listLang->setButtons($btn->name, $btn->name."_cancel");
        $listCover->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function modeServer2() {
        /* @var $faxSession FaxSession */ global $faxSession;
        $this->currentSection = FaxConfigConsts::modeServer2;
        $this->addSection($this->currentSection, 1, true, tl::tl("Mail Configuration"));
        
        $server = $faxSession->faxServer;
        $this->addField($host = new InputPageHiddenIdField("mailhost", $server->mailHost));
        $host->hint = tl::tl("Your email server address.");
        $host->setJavaScriptInit("function JavaScriptInit_{$host->name}() { setFocus('{$host->name}'); }");
        $this->addField($port = new InputPageHiddenIdField("mailport", $server->mailPort));
        $port->hint = tl::tl("Standard port for POP3 is 995.");
        $port->placeHolder = 995;
        $port->emptyOk = true;
        $this->addField($user = new InputPageLoginStringField("mailuser", tl::tl("User"), $server->mailUser));
        $user->hint = tl::tl("The user for the SMTP authorization.") . ' ' . tl::tl("It must be unique for all fax servers.");
        $user->setJavaScriptInit("function JavaScriptInit_{$user->name}() { setFocus('{$user->name}'); }");
        $this->addField($pw = new InputPagePasswordField("mailpw", tl::tl("Password"), $server->mailPassword));
        $pw->hint = tl::tl("The password for the SMTP authorization.");
        $pw->minlength = 0;
        $this->addField($folder = new InputPageHiddenIdField("mailfolder", $server->mailFolder));
        $folder->hint = tl::tl("The inbox folder of the email account.");
        $folder->placeHolder = "INBOX";
        $this->addField($faxServerDomain = new InputPageStringField("faxdomain", tl::tl("Fax Server Domain"), $server->faxServerDomain));
        $faxServerDomain->hint = tl::tl("The domain of the email address of the fax server.");
        $this->addField($admin = new InputPageMailField("mailadmin", tl::tl("Administrator Mail Address"), $server->mailAdmin));
        $this->addField($mailSAPCode = new InputPageCheckboxField("mailsapcode", tl::tl("SAP support")));
        $mailSAPCode->hint = tl::tl("Tick this for SAP support. The SAP result code is appended to the response mail.");
        $hostVal = InputPage::formInput($host->name);
        if(!isset($hostVal)) {
            $mailSAPCode->default["on"] = $server->mailSAPCode;
        }
        
        $this->addField($btn = new ButtonServerConfig2("saveserverconfig", FaxConfigConsts::pageConfig, $this->parent, FaxConfigConsts::modeServer3, FaxConfigConsts::modeServer1));
        $host->setButtons($btn->name, $btn->name."_cancel");
        $port->setButtons($btn->name, $btn->name."_cancel");
        $user->setButtons($btn->name, $btn->name."_cancel");
        $pw->setButtons($btn->name, $btn->name."_cancel");
        $folder->setButtons($btn->name, $btn->name."_cancel");
        $faxServerDomain->setButtons($btn->name, $btn->name."_cancel");
        $admin->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function modeServer3() {
        /* @var $faxSession FaxSession */ global $faxSession;
        /* @var $mainSession MainSession */ global $mainSession;
        $this->currentSection = FaxConfigConsts::modeServer3;
        $this->addSection($this->currentSection, 1, true, tl::tl("PBX Configuration"));
        
        $server = $faxSession->faxServer;
        $this->addField($host = new InputPageStringField("soaphost", tl::tl("PBX IP address"), $server->soapHost));
        $host->hint = tl::tl("The IP address of the master PBX.");
        $host->setJavaScriptInit("function JavaScriptInit_{$host->name}() { setFocus('{$host->name}'); }");
        $this->addField($port = new InputPageDigitstringField("soapport", tl::tl("Port"), $server->soapPort));
        $port->emptyOk = true;
        $port->hint = tl::tl("Just needed, if you use a non standard http(s) port on your device.");
        $this->addField($dynPbx = new InputPageStringField("soapdynpbx", "Dyn-PBX-ID", $server->soapDynPbx));
        $dynPbx->emptyOk = true;
        $dynPbx->hint = tl::tl("Enter the ID of the dynamic PBX, if you use one.");
        $this->addField($user = new InputPageStringField("soapuser", tl::tl("User"), $server->soapUser));
        $user->hint = tl::tl("The name (H.323) of the PBX SOAP user.");
        $this->addField($pw = new InputPagePasswordField("soappw", tl::tl("Password"), $server->soapPassword));
        $pw->hint = tl::tl("The password of the above user.");
        $pw->minlength = 0;
        $this->addField($https = new InputPageCheckboxField("https", "HTTPS"));
        $hostVal = InputPage::formInput($host->name);
        if(!isset($hostVal)) {
            $https->default["on"] = $server->https;
        }
        
        $this->addField($faxUser = new InputPageStringField("soapfaxuser", tl::tl("Fax User"), $server->soapFaxUser));
        $faxUser->hint = tl::tl("The PBX gateway object for fax.");

        $this->addField($btn = new ButtonTestPbx("testpbx", tl::tl("Connection Test")));

        $this->addField($natInfo = new InputPageText("natinfo", "<br>" .
        tl::tl("If there is a NAT router between the Linux AP and the PBX, the NAT IP address and the forwarded port is to be configured here.") . ' ' .
        tl::tl("The Linux AP WebDAV has to be reachable by the PBX with this IP address and port.") . "<br>"));
        $natInfo->attributes["style"] = "margin-top:10px;margin-bottom:10px;max-width:550px";
            
        $this->addField($linuxIP = new InputPageStringField("linuxip", tl::tl("Linux NAT IP address"), $server->linuxIP));
        $linuxIP->hint = tl::tl("The IP address of this device.") . " " . tl::tl("Leave it empty if the local address should be used.");
        $linuxIP->emptyOk = true;
        $linuxIP->setJavaScriptInit("function JavaScriptInit_{$linuxIP->name}() { setFocus('{$linuxIP->name}'); }");
        
        $this->addField($linuxPort = new InputPageDigitstringField("linuxport", tl::tl("Linux NAT port"), $server->linuxPort));
        $linuxPort->hint = tl::tl("The port which is forwarded by the NAT to the HTTP/HTTPS port.")  . " " . tl::tl("Leave it empty if the default port should be used.");
        $linuxPort->placeHolder = 80;
        $linuxPort->emptyOk = true;

        $globalHttps = $mainSession->webCfg->forceHttps ? "true" : "false";
        $localIP = get_dhcp_address();
        $java = "updateWebDAVInfo($globalHttps, '$localIP');";
        $https->attributes["onChange"] = "javascript:$java";
        $linuxIP->attributes["onChange"] = "javascript:$java";
        $linuxPort->attributes["onChange"] = "javascript:$java";
        
        $server->updateWebdavURL();
        $this->addField($webdavInfo = new InputPageTextInfo("webdavinfo", tl::tl("WebDAV URL"), $server->webdavURL));
        $webdavInfo->hint = tl::tl("The HTTP client authentication URL in the PBX must exactly match with the WebDAV URL.");
        $webdavInfo->attributes["style"] = "margin-top:10px;margin-bottom";
        
        $this->addField($btn = new ButtonServerConfig3("saveserverconfig", FaxConfigConsts::pageConfig, $this->parent, FaxConfigConsts::modeServer4, FaxConfigConsts::modeServer2));
        $host->setButtons($btn->name, $btn->name."_cancel");
        $user->setButtons($btn->name, $btn->name."_cancel");
        $pw->setButtons($btn->name, $btn->name."_cancel");
        $https->setButtons($btn->name, $btn->name."_cancel");
        $faxUser->setButtons($btn->name, $btn->name."_cancel");
        $port->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function modeServer4() {
        /* @var $faxSession FaxSession */ global $faxSession;
        $this->currentSection = FaxConfigConsts::modeServer4;
        $this->addSection($this->currentSection, 1, true, tl::tl("Fax Configuration"));
        
        $server = $faxSession->faxServer;
        $this->addField($subscriberId = new FaxInputPagePhoneField("subscriberid", tl::tl("Subscriber Id"), $server->faxSubscriberId));
        $subscriberId->hint = tl::tl("The subscriber Id of the T.30 protocol, which will be sent with a fax.");
        $subscriberId->setJavaScriptInit("function JavaScriptInit_{$subscriberId->name}() { setFocus('{$subscriberId->name}'); }");
        $subscriberId->emptyOk = true;
        $this->addField($addUserNumber = new InputPageCheckboxField("addusernumber", tl::tl("Add User Number")));
        $addUserNumber->hint = tl::tl("If ticked, the user number is appended to the subscriber Id.");
        $this->addField($userNumberPrefix = new InputPageStringField("usernumberprefix", tl::tl("User Number Prefix"), $server->faxUserNumberPrefix));
        $userNumberPrefix->hint = tl::tl("The prefix is added to find the PBX user and removed from the user number for outgoing calls.");
        $userNumberPrefix->emptyOk = true;
        $subId = InputPage::formInput($subscriberId->name);
        if(!isset($subId)) {
            $addUserNumber->default["on"] = $server->faxAddUserNumber;
        }
        $this->addField($subscriberName = new InputPageStringField("subscribername", tl::tl("Subscriber Name"), $server->faxSubscriberName));
        $subscriberName->hint = tl::tl("The subscriber name which will be printed within the header line of each fax page.");
        $subscriberName->emptyOk = true;

        $this->addField($subscriberIdAlt = new FaxInputPagePhoneField("subscriberidalt", tl::tl("Alternative Subscriber Id"), $server->faxSubscriberIdAlt));
        $subscriberIdAlt->hint = tl::tl("The alternative subscriber Id of the T.30 protocol, which will be used if the user is a PBX Map object.");
        $subscriberIdAlt->setJavaScriptInit("function JavaScriptInit_{$subscriberIdAlt->name}() { setFocus('{$subscriberIdAlt->name}'); }");
        $subscriberIdAlt->emptyOk = true;

        $this->addField($maxCallTries = new InputPageDigitstringField("maxcalltries", tl::tl("Max Call Tries"), $server->faxMaxCallTries));
        $maxCallTries->hint = tl::tl("The maximum number of call tries, if a call is not successfull. Calls are aborted because of no available channel aren't counted.");
        $this->addField($resendDelay = new InputPageDigitstringField("resenddelay", tl::tl("Resend Delay"), $server->faxResendDelay));
        $resendDelay->hint = tl::tl("Delay in minutes after which a fax is resend.");
        $this->addField($responseTimeout = new InputPageDigitstringField("responsetimeout", tl::tl("Response Timeout"), $server->faxResponseTimeout));
        $responseTimeout->hint = tl::tl("The response timeout in seconds.");
        $this->addField($retryTimeout = new InputPageDigitstringField("retrytimeout", tl::tl("Retry Timeout"), $server->faxRetryTimeout));
        $retryTimeout->hint = tl::tl("The retry timeout in minutes if a call is retried because of no available channel.");

        $forwardedFileFormats = array(Faxserver::fileFormatPDF => "PDF",
                                      FaxServer::fileFormatTIFF => "TIFF");
        $this->addField($forwardedFileFormat = new InputPageDropdownField("forwardedfileformat", tl::tl("File format of forwarded fax document"), $forwardedFileFormats));
        $forwardedFileFormat->hint = tl::tl("The file format of the forwarded incoming fax document can be chosen here.");
        $forwardedFileFormat->default[$server->faxForwardedFileFormat] = true;
        
        $this->addField($coverIncludesUserText = new InputPageCheckboxField("coverincludesusertext", tl::tl("Cover page includes user's text")));
        $coverIncludesUserText->hint = tl::tl("Tick this if the content of the user's mail is to be included in the fax cover page.");
        if(!isset($subId)) {
            $coverIncludesUserText->default["on"] = $server->faxCoverIncludesUserText;
        }

        $this->addField($confirmationWithDoc = new InputPageCheckboxField("confirmationwithdoc", tl::tl("Confirmation with sent document")));
        $confirmationWithDoc->hint = tl::tl("Tick this if the sent document (as PDF file) is to be included in the confirmation email.");
        if(!isset($subId)) {
            $confirmationWithDoc->default["on"] = $server->faxConfirmationWithDoc;
        }
        $speeds = array(Faxserver::modemSpeedV17 => "V.17, V.29, V.27 (<= 14.400 bit/s)",
                        FaxServer::modemSpeedV29 => "V.29, V.27 (<= 9.600 bit/s)",
                        FaxServer::modemSpeedV27 => "V.27 (<= 4.800 bit/s)");
                
        $this->addField($modemSpeed = new InputPageDropdownField("modemspeed", tl::tl("Modem speed"), $speeds));
        $modemSpeed->default[$server->modemSpeed] = true;
        
        $this->addField($btn = new ButtonServerConfig4("saveserverconfig", FaxConfigConsts::pageConfig, $this->parent, "", FaxConfigConsts::modeServer3));
        $subscriberId->setButtons($btn->name, $btn->name."_cancel");
        $addUserNumber->setButtons($btn->name, $btn->name."_cancel");
        $userNumberPrefix->setButtons($btn->name, $btn->name."_cancel");
        $subscriberName->setButtons($btn->name, $btn->name."_cancel");
        $subscriberIdAlt->setButtons($btn->name, $btn->name."_cancel");
        $maxCallTries->setButtons($btn->name, $btn->name."_cancel");
        $resendDelay->setButtons($btn->name, $btn->name."_cancel");
        $responseTimeout->setButtons($btn->name, $btn->name."_cancel");
        $retryTimeout->setButtons($btn->name, $btn->name."_cancel");
    }

    public static function saveValuesPage3(InputPage &$page) {
        /* @var $faxSession FaxSession */ global $faxSession;
        $host = $page->getField("soaphost");
        $port = $page->getField("soapport");
        $user = $page->getField("soapuser");
        $pw = $page->getField("soappw");
        $faxUser = $page->getField("soapfaxuser");
        $dynPbx = $page->getField("soapdynpbx");
        $https = $page->getField("https");
        $linuxIP = $page->getField("linuxip");
        $linuxPort = $page->getField("linuxport");
        $webdavInfo = $page->getField("webdavinfo");

        if($port->value != "" && !$port->valid() ||
            $linuxPort->value != "" && !$linuxPort->valid()) {
            return false;
        }

        $server = $faxSession->faxServer;
        $server->https = ($https->value["on"] ? true : false);
        $server->soapHost = $host->value;
        $server->soapPort = $port->value;
        $server->soapUser = $user->value;
        $server->soapPassword = $pw->value;
        $server->soapFaxUser = $faxUser->value;
        $server->soapDynPbx = $dynPbx->value;
        $server->linuxIP = $linuxIP->value;
        $server->linuxPort = $linuxPort->value && $linuxPort->value != 0 ? $linuxPort->value : "";
        
        $faxSession->updateServer($server);
        $server->updateWebdavURL();
        $webdavInfo->info = $server->webdavURL;
        return true;
    }
    
   
    function fillServerList() {
        $list = $this->parent->getField("listserver");
        if(!is_object($list)) return;
        $rows = 0;
        
        $faxDb = get_fax_db();
        $query = "SELECT id, name, description FROM servers";   
        $result = $faxDb->query($query);
        while($row = pg_fetch_object($result)) {
            $list->value[$rows][$list->name."name"] = "<a id=\"".htmlentities_utf8($row->name)."\" class=\"myinno-listlink\" href=\"javascript:submitServer('{$row->id}', '".FaxConfigConsts::modeServer1."')\">".htmlentities_utf8($row->name)."</a>";
            $list->value[$rows][$list->name."desc"] = $row->description;
            $list->value[$rows][$list->name."strings"] = "<a class=\"myinno-listlink\" href=\"javascript:submitServer('{$row->id}', '".FaxConfigConsts::modeServerStrings1."')\">".tl::tl("Texts")."</a>";
            $list->value[$rows][$list->name."covers"] = "<a class=\"myinno-listlink\" href=\"javascript:submitServer('{$row->id}', '".FaxConfigConsts::modeServerCovers."')\">".tl::tl("Covers")."</a>";
            $list->value[$rows][$list->name."id"] = $row->id;
            $rows++;
        }
    }
    
    function fillLocationList() {
        $list = $this->parent->getField("listlocations");
        $rows = 0;
        $count = 0;
        /* @var $faxSession FaxSession*/ global $faxSession;
        $server = $faxSession->faxServer;
        $faxDb = get_fax_db();
        $selects = $faxDb->getLanguages($server);
        $covers = $server->getCovers(true);
        
        if(!$list->sent && count($server->locations)) {
            foreach($server->locations as $loc) {
                $list->value[$rows][$list->name."name"] = $loc->name;
                $list->value[$rows][$list->name."lang"][$loc->lang] = true;
                $list->value[$rows][$list->name."lang_options"] = $selects;
                $list->value[$rows][$list->name."cover"][$loc->cover] = true;
                $list->value[$rows][$list->name."cover_options"] = $covers;
                $rows++;
            }
        }
        else if(!count($list->value)) {
            $list->value[$rows][$list->name."name"] = "";
            $list->value[$rows][$list->name."lang"]["en"] = true;
            $list->value[$rows][$list->name."lang_options"] = $selects;
            $list->value[$rows][$list->name."cover"][FaxConfig::coverDefault] = true;
            $list->value[$rows][$list->name."cover_options"] = $covers;
        }
        else {
            unset($vals);
            foreach($list->value as $key => &$vals) {
                $vals[$list->name."lang_options"] = $selects;
                $vals[$list->name."cover_options"] = $covers;
                $rows++;
            }
        }
    }
    
    public function fillStringList() {
        /* @var $faxSession FaxSession*/ global $faxSession;
        /* @var $faxServer FaxServer*/ $faxServer = $faxSession->faxServer;
        $list = $this->parent->getField("liststrings");
        $value = $list->value;
        $list->value = array();
        $rows = 0;
        $strings = FaxServerStrings::getStringIds();
        foreach($strings as $stringConst => $stringId) {
            $list->value[$rows][$list->name."name"] = $stringConst;
            $list->value[$rows][$list->name."value"] = $faxServer->getString($stringId, $faxServer->stringLanguage);
            $list->value[$rows][$list->name."id"] = $stringId;
            $rows++;
        }
        foreach($value as $val) {
            foreach($list->value as &$listVal) {
                if($val[$list->name."id"] == $listVal[$list->name."id"]) {
                    $listVal[$list->name."value"] = $val[$list->name."value"];
                    break;
                }
            }
        }
    }
    
    public function fillCoverList() {
        /* @var $faxSession FaxSession*/ global $faxSession;
        $list = $this->parent->getField("listcovers");
        $covers = $faxSession->faxServer->getCovers();
        $list->value = array();
        $rows = 0;
        foreach($covers as $coverPath => $cover) {
            $list->value[$rows][$list->name."name"] = htmlentities_utf8($cover);
            $del = "";
            if($coverPath != FaxConfig::coverDefault) {
                $del = "<a class=\"inno-tab-table-content\" href=\"javascript:submitCover({$faxSession->faxServer->id}, '$coverPath', '".FaxConfigConsts::modeServerCoversDelete."')\">".tl::tl("Delete")."</a>";
            }
            $list->value[$rows][$list->name."delete"] = $del;
            $list->value[$rows][$list->name."download"] = "<a class=\"novisit\" href=\"#\" onClick=\"javascript:downloadCover('$coverPath', '".self::btnDownloadName."');\">". tl::tl("Download") . "</a>";
            $rows++;
        }
    }
    
    public function fillLists() {
        switch($this->parent->currentTab) {
            case FaxConfigConsts::tabPgsql:
                $this->fillIpList("innovaphone-faxserver");
                break;
            case FaxConfigConsts::tabServer:
                switch($this->parent->currentMode) {
                    case "":
                        $this->fillServerList();
                        break;
                    case FaxConfigConsts::modeServer1:
                        $this->fillLocationList();
                        break;
                    case FaxConfigConsts::modeServerStrings2:
                        $this->fillStringList();
                        break;
                    case FaxConfigConsts::modeServerCovers:
                        $this->fillCoverList();
                        break;
                }
                break;
            case FaxConfigConsts::tabAuthorizedHosts:
                $this->currentSection = $this->parent->currentMode;
                AuthorizedHostsTab::fillList($this, $this->parent->currentMode);
                break;
        }
    }
}

final class FaxConfigPage extends FaxInputPage {
    var $serverId = 0;
    var $serverName = "";
    var $coverName = "";
    
    // constructor
    public function FaxConfigPage() {
        parent::FaxInputPage(tl::tl("Faxserver Config"), "faxconfig");
        global $faxSession;
        $this->setFieldSectionAttributes("inline", new _InputPageFieldSectionAttributes(array(), 1));
        
        // dummy fields to save certain parameters over all pages
        $this->addField(new InputPageHiddenIdField("server_id", $this->serverId));
        $this->serverId = InputPage::formInput("server_id");
        if(!isset($this->serverId)) {
            $this->serverId = 0;
        }
        else {
            $this->serverId = intval($this->serverId);
        }
        
        switch($this->currentMode) {
            case FaxConfigConsts::modeServerCovers:
            case FaxConfigConsts::modeServerCoversAdd:
            case FaxConfigConsts::modeServerCoversDelete:
                $this->addField(new InputPageHiddenIdField("cover_name", $this->coverName));
                $this->coverName = InputPage::formInput("cover_name");
                if(!isset($this->coverName)) $this->coverName = "";
                break;
        }
        
        switch($this->currentMode) {
            case "":
                $faxSession->updateServer(new FaxServer());
                break;
            case FaxConfigConsts::modeServer1:
            case FaxConfigConsts::modeServer2:
            case FaxConfigConsts::modeServer3:
            case FaxConfigConsts::modeServer4:
            case FaxConfigConsts::modeServerStrings1:
            case FaxConfigConsts::modeServerStrings2:
            case FaxConfigConsts::modeServerCovers:
            case FaxConfigConsts::modeServerCoversAdd:
            case FaxConfigConsts::modeServerCoversDelete:
            case FaxConfigConsts::modeDeleteServer:
                if($faxSession->faxServer->id != $this->serverId) {
                    if($this->serverId > 0) {
                        $server = new FaxServer();
                        $server->read("", $this->serverId, get_fax_db());
                        if($server->id > 0) {
                            $faxSession->updateServer($server);
                        }
                        else {
                            $faxSession->updateServer(new FaxServer());
                            $this->serverId = 0;
                        }
                    }
                    else {
                        $faxSession->updateServer(new FaxServer());
                    }
                }
                break;
        }
        $this->serverName = $faxSession->faxServer->name;
        // add proper fields to pages
        $this->pages = array(FaxConfigConsts::pageConfig => new PageConfig($this, false, tl::tl("Faxserver Config")));
        
        // update page
        $this->setDefaultTabProperties();
        $this->pages[$this->pageNo]->createFields();
    }
    
    public function getArgs() {
        $id = "";
        if($this->serverId > 0) {
            $id = "server_id={$this->serverId}&";
        }
        $cover = "";
        if($this->coverName != "") {
            $cover = "cover_name=".urlencode($this->coverName)."&";
        }
        return "{$id}{$cover}tab={$this->currentTab}";
    }
};

// create page
$page = new FaxConfigPage;

if($page->pageNo == FaxConfigConsts::pageConfig) {
    $page->updateFields();
    $page->pages[$page->pageNo]->fillLists();
}

$page->work();
?>