<?php

//  
// innovaphone fax PBX poll PHP script
// copyright (c) innovaphone 2012 - 2015
//

$path=dirname(__FILE__);
require_once("$path/../app-classes/innovaphone.fax.config.class.php");
require_once("$path/../app-classes/innovaphone.fax.class.php");

$version = file_get_contents("$path/../version");

declare(ticks = 1);
?>

innovaphone fax PBX poll PHP script
copyright (c) innovaphone 2012 - 2015

<?php

print "Version " . $version . "\n\n";

$help = false;
foreach($argv as $arg) {
    if(in_array($arg, array('--help', '-help', '-h', '-?'))) {
        $help = true;
    }
}

if(!is_array($_GET) && ($argc < 2 || $help)) {
?>
    Usage:
  <?php echo $argv[0]; ?> <id>

<id> is the configuration database id specific for this server instance.

With the --help, -help, -h, or -? options, you can get this help.

<?php
exit(1);
} 

$configurationId        = null;
if($argc >= 1) {
    $configurationId    = intval($argv[1]);
}
else if(is_array($_GET)) {
    $configurationId    = intval($_GET['id']);
}


setlocale(LC_CTYPE, "en_US.UTF-8");

function getLogDate() {
    $now = new DateTime();
    return date('Ymd-His') . "  -  ";
}

function sig_handler($nsig)
{
    print "\n";
    print getLogDate() . "terminated($nsig).\n";
    exit;
}

pcntl_signal(SIGTERM, "sig_handler", false);
pcntl_signal(SIGINT, "sig_handler", false);
pcntl_signal(SIGHUP, "sig_handler", false);


function terminateSession(&$pbx) {
    try {
        $p = $pbx->End($pbx->getSession());
    }
    catch(Exception $e) {
        return false;
    }
    return true;
}

function shutdown_function() {
    global $pbx;
    if($pbx && $pbx->getKey()) {
        terminateSession($pbx);
    }
    print "\n";
}

register_shutdown_function('shutdown_function');

$pbx    = null;

$sleepTimeout = 0;  
function faxsleep() {
    global $sleepTimeout;
    if($sleepTimeout > 0) {
        sleep($sleepTimeout);
    }
    if($sleepTimeout == 0) {
        $sleepTimeout = 5;          // start with 5 seconds
    }
    else {
        if($sleepTimeout < 60) {    // increase until 60 seconds
            $sleepTimeout++;        
        }
    }
}


do {
    global $sleepTimeout;
    faxsleep();
    print getLogDate() . "Reading configuration... ";
    $cfg     = new FaxServer();
    $cfg->read("", $configurationId);
    if(!$cfg) {
        print "failed.\n";
        print "ERROR: Could not read configuration.\n";
        continue;
    }
    print "done.\n";


    print getLogDate() . "Attaching to shared memory... ";
    $faxShm     = new InnovaphoneFaxShm($cfg->shmId);
    if(!$faxShm->isAttached()) {
        print "ERROR: Could not attach to the SHM.\n";
        continue;
    }
    print "done.\n";

    if(!$faxShm->clear()) {
        print "Couldn't clear SHM calls.\n";
    }
    

    $server     = $cfg->soapHost;
    $port       = $cfg->soapPort;
    $https      = $cfg->https;
    $httpu      = $cfg->soapUser;
    $httpp      = $cfg->soapPassword;
    $userName   = $cfg->soapFaxUser;
    $dynPbxId   = $cfg->soapDynPbx;
    $webdavUrl  = $cfg->webdavURL;
    
    print getLogDate() . "Connecting to PBX " . $server . "... ";
    $pbx = new InnovaphonePBX(
        $server,
        $port,
        $https,
        $httpu,
        $httpp,
        $userName,
        "innovaphone PHP fax polling [" . $version . "]",
        $cfg->trace ? array("trace" => 1) : null,
        dirname(__FILE__) . '/../wsdl/pbx10_00.wsdl',
        $dynPbxId);
        
    if($pbx->getKey() == 0) {
        print "ERROR: Could not login to PBX. Cause: " . $pbx->getFaultString() . "\n";
        continue;
    }
    print "connected.\n\n";
    $pbx->showVersion();
    print "\n";

    $userHandle = 0;
    
    try {
        $userHandle = $pbx->UserInitialize($pbx->getSession(), $userName, true, false, "");
    }
    catch(Exception $e) {
        print getLogDate() . "Reconnecting...\n\n";
        continue;
    }
    if(!$userHandle) {
        print "ERROR: Could not subscribe to user {$userName}.\n";
        continue;
    }

    $fax = new InnovaphoneFax($pbx, $userName);
    $pbxInfo = &$fax;

    if(!$pbxInfo->initialize()) {
        continue;
    }

    $user = $pbxInfo->getUser($userName);
    if(!$user) {
        print "ERROR: User is not in poll list.\n";
        continue;
    }

    $fax->handleNewIncomingCalls($userHandle, $configurationId, $cfg->faxUserNumberPrefix, $faxShm);
    $pbxInfo->purge();

    $checkShmTimeout    = new DateTime();
    $checkShmTimeout->modify("+ 600 seconds");
    
    do {
        try {
            $p = $pbx->Poll($pbx->getSession());
        }
        catch(Exception $e) {
            try {
                $p = $pbx->Echo($pbx->getSession(), $pbx->getKey());
            }
            catch(Exception $e) {
                $pbx = 0;
                print "ERROR: Connection error: " . $e->faultstring . "\n";
                break;
            }
            if($p) {
                continue;
            }
            else {
                $pbx = 0;
                print getLogDate() . "Reconnecting...\n\n";
                break;
            }
        }

        $pbxInfo->update($p);
        $fax->handleNewIncomingCalls($userHandle, $configurationId, $cfg->faxUserNumberPrefix, $faxShm);

        $callList   = $pbxInfo->findUserCalls($userHandle, true);
        foreach($callList as &$call) {
            if(!$call->isActive() && $faxShm) {
                $faxShm->updateCall($call);
            }
        }

        $now = new DateTime();
        if($checkShmTimeout <= $now) {
            $checkShmTimeout    = $now->modify("+ 600 seconds");
            if($faxShm->read() === false) {
                print getLogDate() . "SHM read error.\n\n";
                break;
            }
            foreach($faxShm->getCalls() as $confId => $faxCall) {
                if(!$fax->findCall($confId)) {
                    print getLogDate() . "Remove SHM call $confId.\n";
                    $faxShm->updateCall(new InnovaphoneFaxCall($confId));
                }
            }
        }
        
        $pbxInfo->purge();
        
        $sleepTimeout = 0;  // reset sleep timeout after one successfull run
    } while(true);
    
} while(true);

?>